// COpenGLView.cpp : implementation of the COpenGLView class
//

#include "stdafx.h"
#include "OpenGL View Class.h"

#include "OpenGL View ClassDoc.h"
#include "COpenGLView.h"
#include "math.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif



const char* const COpenGLView::_ErrorStrings[]= {
				{"No Error"},					// 0
				{"Unable to get a DC"},			// 1
				{"ChoosePixelFormat failed"},	// 2
				{"SelectPixelFormat failed"},	// 3
				{"wglCreateContext failed"},	// 4
				{"wglMakeCurrent failed"},		// 5
				{"wglDeleteContext failed"},	// 6
				{"SwapBuffers failed"},			// 7

		};

/////////////////////////////////////////////////////////////////////////////
// COpenGLView

IMPLEMENT_DYNCREATE(COpenGLView, CView)

BEGIN_MESSAGE_MAP(COpenGLView, CView)
	//{{AFX_MSG_MAP(COpenGLView)
	ON_WM_CREATE()
	ON_WM_DESTROY()
	ON_WM_ERASEBKGND()
	ON_WM_SIZE()
	ON_WM_MOUSEMOVE()
	ON_COMMAND(ID_SURFACE, OnSurface)
	ON_UPDATE_COMMAND_UI(ID_SURFACE, OnUpdateSurface)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// COpenGLView construction/destruction

COpenGLView::COpenGLView() :
	 m_hRC(0), m_pDC(0), m_ErrorString(_ErrorStrings[0])

{
	// TODO: add construction code here
	starttime = clock();
	bSurface=false;

}


COpenGLView::~COpenGLView()
{
}

BOOL COpenGLView::PreCreateWindow(CREATESTRUCT& cs) 
{
	// TODO: Add your specialized code here and/or call the base class

	// An OpenGL window must be created with the following flags and must not
    // include CS_PARENTDC for the class style. 
    cs.style |= WS_CLIPSIBLINGS | WS_CLIPCHILDREN;
  	
	return CView::PreCreateWindow(cs);
}


/////////////////////////////////////////////////////////////////////////////
// COpenGLView drawing


void COpenGLView::OnDraw(CDC* pDC)
{
	COpenGLViewClassDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	// TODO: add draw code for native data here

	::glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

	PreRenderScene();


	::glPushMatrix();
	RenderScene();
	::glPopMatrix();

	::glFinish();

	if ( FALSE == ::SwapBuffers( m_pDC->GetSafeHdc() ) )
		{
		SetError(7);
		}
	
	InvalidateRect(0,FALSE);
	GetParent()->PostMessage(WM_PAINT);

}


/////////////////////////////////////////////////////////////////////////////
// COpenGLView diagnostics

#ifdef _DEBUG
void COpenGLView::AssertValid() const
{
	CView::AssertValid();
}

void COpenGLView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

COpenGLViewClassDoc* COpenGLView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(COpenGLViewClassDoc)));
	return (COpenGLViewClassDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// COpenGLView message handlers

int COpenGLView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CView::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	// TODO: Add your specialized creation code here

	InitializeOpenGL();

	return 0;
}


/////////////////////////////////////////////////////////////////////////////
// GL helper functions

void COpenGLView::SetError( int e )
{
	// if there was no previous error,
	// then save this one
	if ( _ErrorStrings[0] == m_ErrorString ) 
		{
		m_ErrorString = _ErrorStrings[e];
		}
}


BOOL COpenGLView::InitializeOpenGL()
{
	// Can we put this in the constructor?
    m_pDC = new CClientDC(this);

    if ( NULL == m_pDC ) // failure to get DC
		{
		SetError(1);
		return FALSE;
		}

	if (!SetupPixelFormat())
		{
        return FALSE;
		}

    //n = ::GetPixelFormat(m_pDC->GetSafeHdc());
    //::DescribePixelFormat(m_pDC->GetSafeHdc(), n, sizeof(pfd), &pfd);

  //  CreateRGBPalette();

    if ( 0 == (m_hRC = ::wglCreateContext( m_pDC->GetSafeHdc() ) ) )
		{
		SetError(4);
		return FALSE;
		}

    if ( FALSE == ::wglMakeCurrent( m_pDC->GetSafeHdc(), m_hRC ) )
		{
		SetError(5);
		return FALSE;
		}	

	// specify black as clear color
    ::glClearColor( 0.0f, 0.0f, 0.0f, 0.0f );
	// specify the back of the buffer as clear depth
    ::glClearDepth( 1.0f );
	// enable depth testing
    ::glEnable( GL_DEPTH_TEST );

	return TRUE;
}


BOOL COpenGLView::SetupPixelFormat()
{
  static PIXELFORMATDESCRIPTOR pfd = 
	{
        sizeof(PIXELFORMATDESCRIPTOR),  // size of this pfd
        1,                              // version number
        PFD_DRAW_TO_WINDOW |            // support window
          PFD_SUPPORT_OPENGL |          // support OpenGL
          PFD_DOUBLEBUFFER,             // double buffered
        PFD_TYPE_RGBA,                  // RGBA type
        24,                             // 24-bit color depth
        0, 0, 0, 0, 0, 0,               // color bits ignored
        0,                              // no alpha buffer
        0,                              // shift bit ignored
        0,                              // no accumulation buffer
        0, 0, 0, 0,                     // accum bits ignored
//        32,                             // 32-bit z-buffer
		16,	// NOTE: better performance with 16-bit z-buffer
        0,                              // no stencil buffer
        0,                              // no auxiliary buffer
        PFD_MAIN_PLANE,                 // main layer
        0,                              // reserved
        0, 0, 0                         // layer masks ignored
    };
    int pixelformat;

    if ( 0 == (pixelformat = ::ChoosePixelFormat(m_pDC->GetSafeHdc(), &pfd)) )
	    {
		SetError(2);
        return FALSE;
		}

    if ( FALSE == ::SetPixelFormat(m_pDC->GetSafeHdc(), pixelformat, &pfd) )
	    {
       	SetError(3);
        return FALSE;
		}

    return TRUE;
}

void COpenGLView::OnDestroy() 
{
	CView::OnDestroy();
	
	// TODO: Add your message handler code here

    if ( FALSE == ::wglDeleteContext( m_hRC ) )
		{
		SetError(6);
 		}

    if ( m_pDC )
		{
        delete m_pDC;
		}
}


BOOL COpenGLView::OnEraseBkgnd( CDC* pDC ) 
{
	// TODO: Add your message handler code here and/or call default
	
	//	return CView::OnEraseBkgnd(pDC);
	return TRUE; // tell Windows not to erase the background
}

void COpenGLView::OnSize(UINT nType, int cx, int cy) 
{
	CView::OnSize(nType, cx, cy);
	
	// TODO: Add your message handler code here
	GLdouble aspect_ratio; // width/height ratio
	
	if ( 0 >= cx || 0 >= cy )
		{
		return;
		}


	SetupViewport( cx, cy );

	// select the projection matrix and clear it
    ::glMatrixMode( GL_PROJECTION );
    ::glLoadIdentity();

	// compute the aspect ratio
	// this will keep all dimension scales equal
	aspect_ratio = (GLdouble)cx/(GLdouble)cy;

	// select the viewing volumn
	SetupViewingFrustum( aspect_ratio );
	
	// switch back to the modelview matrix
    ::glMatrixMode( GL_MODELVIEW );
    ::glLoadIdentity();

	// now perform any viewing transformations
	SetupViewingTransform();
}

 /////////////////////////////////////////////////////////////////////////////
// COpenGLView helper functions

BOOL COpenGLView::SetupViewport( int cx, int cy )
{
	// select the full client area
    ::glViewport(0, 0, cx, cy);

	return TRUE;
}

BOOL COpenGLView::SetupViewingFrustum( GLdouble aspect_ratio )
{
	// select a default viewing volumn
    ::gluPerspective( 40.0f, aspect_ratio, .1f, 20.0f );
	return TRUE;
}


BOOL COpenGLView::SetupViewingTransform()
{
	// select a default viewing transformation
	// of a 20 degree rotation about the X axis
	// then a -5 unit transformation along Z
	::glTranslatef( 0.0f, 0.0f, -5.0f );
	::glRotatef( 20.0f, 1.0f, 0.0f, 0.0f );
    return TRUE;
}


BOOL COpenGLView::RenderScene()
{
	GLfloat vertex1[3];
	GLfloat vertex2[3];
	GLfloat vertex3[3];
	GLfloat vertex4[3];
	GLfloat vertex5[3];
	GLfloat vertex6[3];
	GLfloat vertex7[3];
	GLfloat vertex8[3];
	GLfloat res1[3];
	GLfloat res2[3];
	GLfloat res3[3];
	GLfloat res4[3];

	float u,v, delta_u=0.02f,delta_v=0.01f;
	static float time=-0.1f;
	float delta_time=0.1f;
	if(time<1.5)
		delta_time=0.2;
	if(time<0.7)
		delta_time=0.4;
	time+=delta_time;
	if(time>6) time=0.0;

	if(!bSurface){
		::glDisable(GL_LIGHTING);
		::glColor3f( 0.75f, 0.75f, 1.0f ); //rgb; 1,0,0 = red
		//::glPushMatrix();
		//::glTranslatef(lightx,lighty,lightz);
		//::auxWireSphere(0.1f);
		//::glPopMatrix();
	}
	else{
		::glEnable(GL_LIGHTING);
		::glLightModeli(GL_LIGHT_MODEL_LOCAL_VIEWER, GL_TRUE );

		GLfloat local_amb[]={0.5f,0.5f,0.9f,1.0f};

		::glLightModelfv(GL_LIGHT_MODEL_AMBIENT,local_amb);

		GLfloat pos[]={0.0,2.0,-3.0,0.0f};
		GLfloat amb[]={0.5f,0.5f,0.0f,1.0f};
		GLfloat diff[]={1.0f,0.1f,0.5f,1.0f};
		GLfloat spec[]={1.0f,1.0f,0.0f,1.0f};

		::glEnable(GL_LIGHT0);
	//	::glLightf(GL_LIGHT0,GL_CONSTANT_ATTENUATION ,1.0);
	//
		::glLightfv(GL_LIGHT0,GL_AMBIENT ,amb);
		::glLightfv(GL_LIGHT0,GL_SPECULAR ,spec);
		::glLightfv(GL_LIGHT0,GL_DIFFUSE ,diff);
		::glLightfv(GL_LIGHT0,GL_POSITION ,pos);

		GLfloat mat_shine[]={128.0f};
		GLfloat mat_spec[]={0.60f,0.60f,1.0f,1.0f};
		GLfloat mat_diff[]={0.2f,0.5f,0.9f,1.0f};
		GLfloat mat_amb[]={0.2f,0.5f,0.9f,1.0f};

		::glMaterialfv(GL_FRONT,GL_AMBIENT,mat_amb);
		::glMaterialfv(GL_FRONT,GL_DIFFUSE,mat_diff);
		::glMaterialfv(GL_FRONT,GL_SHININESS,mat_shine);
		::glMaterialfv(GL_FRONT,GL_SPECULAR,mat_spec);
	}


	/*
		GLfloat materialSpecular[4] = { 1.0f, 1.0f, 1.0f, 1.0f };
		GLfloat materialShininess[1] = { 128.0f };
		GLfloat materialAmbient[4] = { 0.25f, 0.25f, 0.25f, 1.0f };
		GLfloat materialDiffuse[4] = { 0.4f, 0.4f, 0.4f, 1.0f };

		GLfloat local_ambient[] = { 0.0f, 0.0f, 0.0f, 1.0f };
    
		::glEnable(GL_DEPTH_TEST);
		::glDepthFunc(GL_LESS);


		::glLightModelfv(GL_LIGHT_MODEL_AMBIENT, local_ambient);

    GLfloat ambient0[] =  { 0.0f, 0.0f, 0.0f, 1.0f };
    GLfloat diffuse0[] =  { 1.0f, 0.0f, 0.0f, 1.0f };
    GLfloat specular0[] = { 1.0f, 0.0f, 0.0f, 1.0f };
    GLfloat position0[] = { 2.0f, -1.5f, -1.5f, 1.0f };

    GLfloat ambient1[] =  { 0.0f, 0.0f, 0.0f, 1.0f };
    GLfloat diffuse1[] =  { 1.0f, 1.0f, 1.0f, 1.0f };
    GLfloat specular1[] = { 0.5f, 0.5f, 0.5f, 1.0f };
    GLfloat position1[] = { 2.0f, 1.0f, -1.0f, 0.0f };

    GLfloat ambient2[] =  { 0.0f, 0.0f, 0.0f, 1.0f };
    GLfloat diffuse2[] =  { 0.0f, 0.0f, 1.0f, 1.0f };
    GLfloat specular2[] = { 0.0f, 0.0f, 1.0f, 1.0f };
    GLfloat position2[] = { -0.5f, -0.5f, -1.0f, 1.0f };

	GLfloat ambient3[] =  { 0.0f, 0.0f, 0.0f, 1.0f };
    GLfloat diffuse3[] =  { 1.0f, 1.0f, 1.0f, 1.0f };
    GLfloat specular3[] = { 0.0f, 0.0f, 0.0f, 1.0f };
    GLfloat position3[] = { lightx, lighty, 0.5f, 0.0f };


	// Dispable lighting to draw some simple spheres that
	// represent the lights
	::glDisable( GL_LIGHTING );

	::glPushMatrix();
	::glColor3fv( diffuse0 );
	::glTranslatef( position0[0], position0[1], position0[2] );
  	::auxWireSphere( 0.15f );

	::glPopMatrix();

	::glPushMatrix();
	::glColor3fv( diffuse1 );
	::glTranslatef( position1[0], position1[1], position1[2] );
//  	::auxWireSphere( 0.15f );
	::glPopMatrix();

	::glPushMatrix();
	::glColor3fv( diffuse2 );
	::glTranslatef( position2[0], position2[1], position2[2] );
	::auxWireSphere( 0.15f );

	::glPopMatrix();

	::glPushMatrix();
	::glColor3fv( diffuse3 );
	::glTranslatef( position3[0], position3[1], position3[2] );
//  	::auxWireSphere( 0.15f );
	::glPopMatrix();

	::glEnable( GL_LIGHTING );

    //::glEnable(GL_LIGHT0);  
    ::glLightfv(GL_LIGHT0, GL_AMBIENT, ambient0);
    ::glLightfv(GL_LIGHT0, GL_POSITION, position0);
    ::glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuse0);
    ::glLightfv(GL_LIGHT0, GL_SPECULAR, specular0);

    //::glEnable(GL_LIGHT1);
    ::glLightfv(GL_LIGHT1, GL_AMBIENT, ambient1);
    ::glLightfv(GL_LIGHT1, GL_POSITION, position1);
    ::glLightfv(GL_LIGHT1, GL_DIFFUSE, diffuse1);
    ::glLightfv(GL_LIGHT1, GL_SPECULAR, specular1);
	
    //::glEnable(GL_LIGHT2);  
    ::glLightfv(GL_LIGHT2, GL_AMBIENT, ambient2);
    ::glLightfv(GL_LIGHT2, GL_POSITION, position2);
    ::glLightfv(GL_LIGHT2, GL_DIFFUSE, diffuse2);
    ::glLightfv(GL_LIGHT2, GL_SPECULAR, specular2);

    //::glEnable(GL_LIGHT3);  
    ::glLightfv(GL_LIGHT3, GL_AMBIENT, ambient3);
    ::glLightfv(GL_LIGHT3, GL_POSITION, position3);
    ::glLightfv(GL_LIGHT3, GL_DIFFUSE, diffuse3);
    ::glLightfv(GL_LIGHT3, GL_SPECULAR, specular3);
 
	::glMaterialfv( GL_FRONT, GL_SPECULAR, materialSpecular );
	::glMaterialfv( GL_FRONT, GL_SHININESS, materialShininess );
	::glMaterialfv( GL_FRONT, GL_DIFFUSE, materialDiffuse );
	::glMaterialfv( GL_FRONT, GL_AMBIENT, materialAmbient );
*/

//	::glTranslatef(0.0,-2.0,0.0);
//	::glRotatef(lighty*50-90,1.0,0.0,0.0);

	if(bSurface)
		::glBegin(GL_TRIANGLES); // each pair of glVertex will form a line
	else
		::glBegin(GL_LINES);

	::glEnable(GL_NORMALIZE);
	::glShadeModel(GL_FLAT);

int trig_count=0;
	//ripples
	for(u=0;u<1;u+=delta_u){
		for(v=0;v<1;v+=delta_v){
			MySurface(u,v,time,vertex1);
			MySurface(u+delta_u,v,time,vertex2);
			MySurface(u,v+delta_v,time,vertex3);
			if(v<delta_v){
				res1[0]=0.0;res2[1]=1.0;res2[2]=0.0;}
			else
				normal(vertex3,vertex1,vertex2,res1);
			glNormal3fv(res1);
			glVertex3fv(vertex1);

			MySurface(u+delta_u+delta_u,v,time,vertex5);
			MySurface(u+delta_u,v+delta_v,time,vertex4);
			if(v<delta_v){
				res2[0]=0.0;res2[1]=1.0;res2[2]=0.0;}
			else
				normal(vertex5,vertex2,vertex4,res2);
			glNormal3fv(res2);
			glVertex3fv(vertex2);

			MySurface(u,v+delta_v+delta_v,time,vertex8);
			normal(vertex4,vertex3,vertex8,res3);
			glNormal3fv(res3);
			glVertex3fv(vertex3);
			
			MySurface(u+delta_u,v+delta_v,time,vertex4);
			MySurface(u+delta_u+delta_u,v+delta_v,time,vertex6);
			MySurface(u+delta_u,v+delta_v+delta_v,time,vertex7);
			normal(vertex6,vertex4,vertex7,res4);

			glNormal3fv(res4);
			glVertex3fv(vertex4);

			glNormal3fv(res2);
			glVertex3fv(vertex2);
			
			glNormal3fv(res3);
			glVertex3fv(vertex3);

			trig_count+=2;


		}
	}
	::glEnd();

/*
	//sphere
	::glEnable(GL_NORMALIZE);
	::glShadeModel(GL_FLAT);
	::glBegin(GL_TRIANGLES); // each pair of glVertex will form a line
	for(u=0;u<1;u+=delta_u){
		for(v=0;v<1;v+=delta_v){
			MySphere(u,v,vertex1);
			MySphere(u+delta_u,v,vertex2);
			MySphere(u,v+delta_v,vertex3);
			normal(vertex2,vertex1,vertex3,res);
			glNormal3fv(res);
			glVertex3fv(vertex3);
			//glNormal3fv(vertex2);
			glVertex3fv(vertex2);
			//glNormal3fv(vertex1);
			glVertex3fv(vertex1);
			MySphere(u+delta_u,v+delta_v,vertex1);
			//glNormal3fv(vertex1);
			//normal(vertex1,vertex2,vertex3,res);
			glVertex3fv(vertex1);
			//glNormal3fv(vertex2);
			glVertex3fv(vertex2);
			//glNormal3fv(vertex3);
			glVertex3fv(vertex3);

		}
	}
	::glEnd();
*/



	static int updateFrame = 10;
	float fps;

	if (++updateFrame <10 )
		return TRUE;

	updateFrame = 0;

	fps=10/((float)(clock()-starttime)/(float)(CLOCKS_PER_SEC));
	starttime=clock();

	char string[256];
	_snprintf( string, 200, "%s ( %6.1f Frames/sec ) ( %d Triangles) (%4.0f Triangles per second)",
		"Ripples", fps ,trig_count,(trig_count*fps));

	GetParentFrame()->SetWindowText( string );
    return TRUE;
}	

#define TWO_PI 2*3.141
#define PI 3.141


void COpenGLView::MySurface(float u,float v,float t,GLfloat* vert){
	float r=v*5;
	float damp=0.5f;
	float freq=5.0f;
	float amp=0.6f;

	vert[0]=r*(float)cos(TWO_PI*u);
	if(r>t)
		vert[1]=0;
	else
		vert[1]=-amp*(exp(-damp*(t-r))*(sin(freq*(t-r))/(r*r+1)));
		
	vert[2]=(r*sin(TWO_PI*u));

/*	
	r=sqrt(vert[2]*vert[2]+((1-vert[0])*(1-vert[0])));
	if(r>t)
		vert[1]+=0;
	else
		vert[1]=(-amp*(exp(-damp*(t-r))*(sin(freq*(t-r))/(r*r+1))));
*/

}

void COpenGLView::MySphere(float u,float v,GLfloat* vert){
	float r=0.5f;

	vert[0]=r*(float)cos(PI*u-PI/2)*(float)cos(TWO_PI*v);
	
	vert[1]=r*(float)sin((PI*u-PI/2));
	
	vert[2]=r*(float)cos(PI*u-PI/2)*(float)sin(TWO_PI*v);
}

void COpenGLView::OnMouseMove(UINT nFlags, CPoint point) 
{
	// TODO: Add your message handler code here and/or call default
	
	lightx=(float)point.x/200.0f-2;
	if(nFlags&MK_LBUTTON)
		lighty=2-(float)point.y/200.0f;
	else
		lightz=2-(float)point.y/200.0f;
	
	CView::OnMouseMove(nFlags, point);
}


GLfloat* COpenGLView::normal(GLfloat a[3],GLfloat b[3],GLfloat c[3],GLfloat r[3]){
	GLfloat x[3],y[3],z[3];

	x[0]=b[0]-a[0];
	x[1]=b[1]-a[1];
	x[2]=b[2]-a[2];

	y[0]=b[0]-c[0];
	y[1]=b[1]-c[1];
	y[2]=b[2]-c[2];

	r[0]=0.0;
	r[1]=1.0;
	r[2]=0.0;

	//return r;

	r[0]=(x[1]*y[2]-y[1]*x[2]);
	r[1]=-(x[0]*y[2]-y[0]*x[2]);
	r[2]=(x[0]*y[1]-y[0]*x[1]);

	GLdouble d= r[0]*r[0]+r[1]*r[1]+r[2]*r[2];

	if(d<(GLdouble)0.00000001)
		d=100000000.0;
	else
		d=1.0/sqrt(d);

	r[0]*=d;
	r[1]*=d;
	r[2]*=d;
	
	//r[0]=a[0];
	//r[1]=a[1];
	//r[2]=a[2];
	return r;
}

void COpenGLView::OnSurface() 
{
	// TODO: Add your command handler code here
	bSurface=!bSurface;

	
}

void COpenGLView::OnUpdateSurface(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	pCmdUI->SetCheck(bSurface);
	
}
